#!/usr/bin/perl
# Actually submit a support ticket

require './virtualmin-support-lib.pl';
&ReadParse();
$main::theme_forbid_nice_size = 1;
&error_setup($text{'ticket_err'});
if (&foreign_check("virtual-server")) {
	&foreign_require("virtual-server", "virtual-server-lib.pl");
	}

# Validate inputs
if (defined($in{'user'})) {
	$in{'user'} =~ /^\S+$/ || &error($text{'ticket_euser'});
	$in{'pass'} =~ /\S/ || &error($text{'ticket_epass'});
	}
$in{'subject'} =~ /\S/ || &error($text{'ticket_esubject'});
$in{'body'} =~ /\S/ || &error($text{'ticket_ebody'});

&ui_print_unbuffered_header(undef, $text{'ticket_title'}, "");

# Save Virtualmin login
if ($in{'remember'}) {
	$config{'ticket_user'} = $in{'user'};
	$config{'ticket_pass'} = $in{'pass'};
	&save_module_config();
	}

if ($in{'validation'}) {
	# Collect validation info
	print $text{'ticket_validating'},"<br>\n";
	&clean_environment();
	$ENV{'WEBMIN_CONFIG'} = $config_directory;
	$ENV{'WEBMIN_VAR'} = $var_directory;
	$validation = &backquote_command("$virtual_server::module_root_directory/validate-domains.pl --all-domains --all-features");
	&reset_environment();

	# Collect config check
	&virtual_server::push_all_print();
	&virtual_server::set_all_text_print();
	($configcheck, $cerrs) = &capture_function_output(
		\&virtual_server::check_virtual_server_config);
	if ($cerr && $cerr->[0]) {
		$cerr->[0] = &virtual_server::html_tags_to_text($cerr->[0]);
		$configcheck .= "ERROR: $cerr->[0]\n";
		}
	&virtual_server::pop_all_print();
	print $text{'enable_done'},"<br><br data-x-br>\n";
	}

if ($in{'details'}) {
	# Collect detected system info
	print $text{'ticket_collecting'},"<br>\n";
	$main::sysinfo_virtualmin_self = 1;
	$info = &virtual_server::get_collected_info('no-cache');

	# No need for full domain detail
	foreach my $q (@{ $info->{'quota'} }) {
	    $q->[0] = $q->[0]->{'dom'};
	    }

	# Print quotas size nicely
	foreach my $q (@{ $info->{'quota'}->[0] }) {
	    my $qn = int($q);
	    if ($qn > 0) {
	        $q = nice_size($qn);
	        }
	    }

	# Print disk sizes nicely
	foreach my $d (@{ $info->{'disk_fs'} }) {
	    foreach my $di ($d) {
	        foreach $k (keys %{$di}) {
	            my $din = int($di->{$k});
	            if ($din > 0) {
	                $di->{$k} = nice_size($din);
	                }
	            }
	        }
	    }

	# Print memory and swap size nicely
	foreach my $m (@{ $info->{'mem'} }) {
	    $m = nice_size($m * 1024);
	    }

	# Print other sizes nicely
	$info->{'maxquota'}  = nice_size($info->{'maxquota'});
	$info->{'disk_free'} = nice_size($info->{'disk_free'});
	$info->{'disk_total'} = nice_size($info->{'disk_total'});

	$collected = &hash_to_text($info);
	print $text{'enable_done'},"<br><br data-x-br>\n";
	}

# Build HTTP request parameters
$pars = "subject=".&urlize($in{'subject'})."&".
	"body=".&urlize($in{'body'})."&".
	"os=".&urlize($in{'os'})."&".
	"version=".&urlize($in{'ver'})."&".
	"webmin=".&get_webmin_version()."&".
	"type=bug&".
	"private=".int($in{'private'})."&";
foreach $mod ("virtual-server", "server-manager", "virtual-server-theme") {
	%minfo = &get_module_info($mod);
	if (!%minfo) {
		%minfo = &get_theme_info($mod);
		}
	if (%minfo) {
		$pars .= "$mod=$minfo{'version'}&";
		}
	}
if ($validation) {
	$pars .= "validation=".&urlize($validation)."&";
	}
if ($configcheck) {
	$pars .= "configcheck=".&urlize($configcheck)."&";
	}
if ($collected) {
	$pars .= "collected=".&urlize($collected)."&";
	}
if (defined($in{'user'})) {
	($serial, $key) = &get_virtualmin_serial();
	# Login by username and password
	$pars .= "username=".&urlize($in{'user'})."&".
		 "password=".&urlize($in{'pass'})."&".
		 "serial=".&urlize($serial)."&".
		 "key=".&urlize($key);
	}
else {
	# By serial and key
	($serial, $key) = &get_virtualmin_serial();
	$pars .= "serial=".&urlize($serial)."&".
		 "key=".&urlize($key);
	}

# Submit the ticket via HTTP
print $text{'ticket_sending'},"<br>\n";
($host, $port, $page, $ssl) = &parse_http_url($virtualmin_ticket_url);
&post_http_connection($host, $port, $page, $pars, \$out, \$err);
if ($err) {
	# HTTP failed
	print &text('ticket_ehttp', $err),"<br><br data-x-br>\n";
	}
elsif ($out =~ /^(ERR|BAD)\s+(.*)/) {
	# Got error back
	print &text('ticket_efailed', "$2"),"<br><br data-x-br>\n";
	}
elsif ($out =~ /^OK\s+(\d+)\s+(\S+)/) {
	# Worked - got ticket ID and URL
	print &text('ticket_created',
		    "<a href='$2' target=_new>#$1</a>"),"<br><br data-x-br>\n";
	&webmin_log('ticket', undef, undef, { 'subject' => $in{'subject'},
					      'bug' => "#$1",
					      'url' => $2 });
	}
else {
	# Unknown error?
	print &text('ticket_eunknown2', $out),"<br><br data-x-br>\n";
	}

&ui_print_footer("", $text{'index_return'});

sub hash_to_text
{
my ($h, $i) = @_;
my $rv;
foreach my $k (keys %$h) {
	my $v = $h->{$k};
	if (ref($v) eq 'HASH') {
		$rv .= $i.$k."={\n";
		$rv .= &hash_to_text($v, $i."  ");
		$rv .= $i."}\n";
		}
	elsif (ref($v) eq 'ARRAY') {
		$rv .= $i.$k."="."[\n";
		$rv .= &array_to_text($v, $i."  ");
		$rv .= $i."]\n";
		}
	else {
		$rv .= $i.$k."=".$v."\n";
		}
	}
return $rv;
}

sub array_to_text
{
my ($a, $i) = @_;
my $rv;
foreach my $v (@$a) {
	if (ref($v) eq 'HASH') {
		$rv .= $i."{\n";
		$rv .= &hash_to_text($v, $i."  ");
		$rv .= $i."}\n";
		}
	elsif (ref($v) eq 'ARRAY') {
		$rv .= $i."[\n";
		$rv .= &array_to_text($v, $i."  ");
		$rv .= $i."]\n";
		}
	else {
		$rv .= $i.$v."\n";
		}
	}
return $rv;
}
